import Vue from 'vue';
import Router from 'vue-router';

import store from '../store';
import HomePage from '../components/home-page';
import Login from '../components/login';
import AudioCall from '../components/audio-call';
import VideoCall from '../components/video-call';

function toQueryPair(key, value) {
  if (typeof value == 'undefined') {
    return `&${key}=`;
  }
  return `&${key}=${value}`;
}

function objToParam(param) {
  if (Object.prototype.toString.call(param) !== '[object Object]') {
    return '';
  }
  let queryParam = '';
  for (let key in param) {
    if (Object.prototype.hasOwnProperty.call(param, key)) {
      let value = param[key];
      queryParam += toQueryPair(key, value);
    }
  }
  return queryParam;
}

Vue.use(Router);

export function createRouter() {
  const router = new Router({
    mode: 'hash',
    fallback: false,
    routes: [
      { path: '/', component: HomePage },
      { path: '/login', component: Login },
      { path: '/audio-call', component: AudioCall },
      { path: '/video-call', component: VideoCall }
    ]
  });
  router.beforeEach((to, from, next) => {
    if (!store.state.isLogin) {
      // console.log('to', objToParam(to.query));
      // console.log('to', to);
      if (Object.prototype.hasOwnProperty.call(to.query, 'veteMemberId')) {
        store.commit("setVeteMemberId", to.query.veteMemberId);
      }
      if (to.path !== '/login') {
        if (from.path !== '/login') {
          next('/login?' + objToParam(to.query));
        }
        return;
      }
    }
    next();
  })
  return router;
}

const originalPush = Router.prototype.push
Router.prototype.push = function push(location) {
  return originalPush.call(this, location).catch(err => err)
}